import React          from 'react'
import style          from './style.scss'
import CSSModules     from 'react-css-modules'
import moment         from 'moment'

import {
  Grid,
  Cell,
  FABButton,
  IconButton,
  Icon,
  Dialog,
  Checkbox,
  Button,
  DialogTitle,
  DialogContent,
  DialogActions,
  Textfield,
  Switch,
  ProgressBar,
  Snackbar
} from 'react-mdl'

class CarmelComponent extends React.Component {

  constructor(props) {
    super(props)

    const lastDay = moment(this.props.carmel.endDate)
    this.state = {
      lastDay: lastDay,
      signups: [],
      signupActive: false,
      sending: false,
      signupDetails: {},
      daysLeft: lastDay.diff(moment(), 'days') + 1,
    }
    moment.locale(this.props.carmel.timeLocale)
  }

  get today() {
    return moment().startOf('day')
  }

  parseTimeSlots(signup) {
    var slots   = {}
    const today = moment().startOf('day')
    const day   = moment(signup.day).startOf('day')
    const dayId = day.diff(today, 'days')

    if (dayId < 0 ) {
      // This is in the past so we don't care anymore
      return slots
    }

    var dayIds = [dayId]

    if (signup.recurring) {
      var nextDay   = day.add(1, 'weeks')
      var nextDayId = nextDay.diff(moment(), 'days') + 1
      var diff      = this.state.lastDay.diff(nextDay, 'days') + 1
      while(diff >= 0) {
        dayIds.push(nextDayId)
        var nextDay   = nextDay.add(1, 'weeks')
        var nextDayId = nextDay.diff(moment(), 'days') + 1
        var diff      = this.state.lastDay.diff(nextDay, 'days') + 1
      }
    }

    dayIds.forEach(nextDayId => {
      // Allocate people for the day, for all hours
      for (let durationId = 0; durationId < signup.duration; durationId++) {
        const time                    = signup.hour + durationId
        slots[nextDayId]              = slots[nextDayId] || {}
        slots[nextDayId][time]        = slots[nextDayId][time] || {}
        slots[nextDayId][time].total  = slots[nextDayId][time].total || 0
        slots[nextDayId][time].total  += signup.people
      }
    })

    return slots
  }

  parseSignups(data) {
    if (!data || !data.signups) {
      return
    }

    var signups = {}
    data.signups.forEach(signup => {
      const signupTimeSlots = this.parseTimeSlots(signup)
      if (signupTimeSlots) {
        const keys = Object.keys(signupTimeSlots)
        keys.forEach(key => {
          signups[key] = Object.assign(signups[key] || {}, signupTimeSlots[key])
        })
      }
    })

    this.setState({ signupActive: false, sending: false, signups: signups })
  }

  fetchSignups() {
    var self = this
    this.props.carmel.apiClient.get("signups").
         then(data => self.parseSignups(data)).
         catch(error => {
           //TODO: handle errors
           console.log(error)
         })
  }

  sendSignup() {
    const day = moment(this.state.signupDetails.data.day).format('YYYYMMDD')
    const data = {
      name: this.state.signupDetails.name,
      email: this.state.signupDetails.email,
      phone: this.state.signupDetails.phone,
      people: this.state.signupDetails.people,
      comments: this.state.signupDetails.comments || "none",
      duration: this.state.signupDetails.duration,
      recurring: this.state.signupDetails.recurrence || false,
      hour: this.state.signupDetails.hour.time,
      day: day
    }

    this.setState({ signupActive: false, sending: true })

    var self = this
    this.props.carmel.apiClient.post("signups", { data }).
         then((result) => {
           self.fetchSignups()
         }).
         catch(error => {
            console.log(error)
         })
  }

  timeStats(data, time) {
    if (!this.state.signups ||
        !this.state.signups[data.dayId] ||
        !this.state.signups[data.dayId][time]) {
      return { total: 0 }
    }

    return this.state.signups[data.dayId][time]
  }

  componentDidMount() {
    this.fetchSignups()
  }

  renderHour(data, hour) {
    return (<div key={hour.time} styleName="hour">
        <div styleName={`info-${hour.type}`}>
          <p> { hour.title } </p>
        </div>
        <div styleName="actions">
          <FABButton mini ripple primary onClick= {this.openSignup.bind(this, { data, hour, duration: 1, people: 1 })}>
            <Icon name="add" />
          </FABButton>
        </div>
    </div>)
  }

  openSignup(details) {
    this.setState({ signupActive: true, signupDetails: details })
  }

  doneSignup() {
    if (!this.state.signupDetails.name) {
      this.setState({ signupError: this.props.carmel.text.nameError, showSignupError: true })
      return
    }

    if (!this.state.signupDetails.email) {
      this.setState({ signupError: this.props.carmel.text.emailError, showSignupError: true })
      return
    }

    if (!this.state.signupDetails.phone) {
      this.setState({ signupError: this.props.carmel.text.phoneError, showSignupError: true })
      return
    }

    this.sendSignup()
  }

  cancelSignup() {
    this.setState({ signupActive: false, sending: false })
  }

  increaseDuration() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { duration: this.state.signupDetails.duration + 1})})
  }

  toggleRecurrence () {
    const oldValue = this.state.signupDetails.recurrence || false
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { recurrence: !oldValue })})
  }

  nameUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { name: event.target.value })})
  }

  emailUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { email: event.target.value })})
  }

  phoneUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { phone: event.target.value })})
  }

  commentsUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { comments: event.target.value })})
  }

  decreaseDuration() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { duration: this.state.signupDetails.duration - 1})})
  }

  increasePeople() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { people: this.state.signupDetails.people + 1})})
  }

  decreasePeople() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { people: this.state.signupDetails.people - 1})})
  }

  onSignupFormAlertTimeout() {
    this.setState({ showSignupError: false, signupError: null })
  }

  renderSignupFormAlert() {
    return (<Snackbar
                active={this.state.showSignupError || false}
                onTimeout={this.onSignupFormAlertTimeout.bind(this)}>
        { this.state.signupError }
    </Snackbar>)
  }

  renderSignupForm() {
    const day =  moment().add(this.state.signupDetails.data.dayId, 'days')
    const date = day.format('dddd, Do MMMM')
    const weekday = day.format('dddd')
    const people = this.state.signupDetails.people
    const duration = this.state.signupDetails.duration
    const minDuration = 1
    const maxDuration = (this.props.carmel.endTime - this.state.signupDetails.hour.time - 1)
    const time = `${this.state.signupDetails.hour.time}:00 - ${this.state.signupDetails.hour.time + this.state.signupDetails.duration}:00`
    const minPeople = 1
    const maxPeople = 999999

    return (<div styleName="formContent">
      <p> { `${date}, ${time}` } </p>

      <div styleName="formOptions">
        <IconButton name="schedule" /> { this.props.carmel.text.duration }
        <IconButton styleName="formOption" disabled={this.state.signupDetails.duration <= minDuration} name="remove" onClick={this.decreaseDuration.bind(this)}/>
          { `${duration}` }
        <IconButton styleName="formOption" disabled={this.state.signupDetails.duration > maxDuration} name="add" onClick={this.increaseDuration.bind(this)}/>
        <br/>
          <IconButton name="group" /> { this.props.carmel.text.people }
          <IconButton styleName="formOption" disabled={this.state.signupDetails.people <= minPeople} name="remove" onClick={this.decreasePeople.bind(this)}/>
            { `${people}` }
          <IconButton styleName="formOption" disabled={this.state.signupDetails.people > maxPeople} name="add" onClick={this.increasePeople.bind(this)}/>
          <br/>
        <Switch styleName="formOption" ripple id="switch1" onChange={this.toggleRecurrence.bind(this)}>
          { `${this.props.carmel.text.everyWeekday} ${weekday}` }
        </Switch>
      </div>

      <Textfield
          styleName="formField"
          floatingLabel
          onChange={this.nameUpdate.bind(this)}
          label={this.props.carmel.text.name}/>
      <Textfield
          styleName="formField"
          floatingLabel
          onChange={this.emailUpdate.bind(this)}
          label={this.props.carmel.text.email}/>
      <Textfield
        styleName="formField"
        floatingLabel
        onChange={this.phoneUpdate.bind(this)}
        label={this.props.carmel.text.phone}/>
      <Textfield
        styleName="formField"
        floatingLabel
        rows={3}
        onChange={this.commentsUpdate.bind(this)}
        label={this.props.carmel.text.comments}/>

        { this.renderSignupFormAlert() }

      </div>)
  }

  renderSignup() {
    return (<Dialog open={this.state.signupActive} style={{ width: "80%" }}>
      <DialogContent styleName="form">
        { this.renderSignupForm() }
      </DialogContent>
      <DialogActions styleName="formActions">
        <Button styleName="formActionButton" raised colored onClick={ this.doneSignup.bind(this) }> { this.props.carmel.text.ok } </Button>
        <Button styleName="formActionButton" raised onClick={ this.cancelSignup.bind(this) }> { this.props.carmel.text.cancel } </Button>
      </DialogActions>
    </Dialog>)
  }

  renderProgress() {
    return (<div styleName="progress">
      <h2> {this.props.carmel.text.progress} </h2>
    </div>)
  }

  hours(data) {
    var elements = []
    const startTime = this.props.carmel.startTime
    const endTime   = this.props.carmel.endTime
    for (var time   = startTime; time < endTime; time++) {
      const stats   = this.timeStats(data, time)
      const title   = `${time}:00`
      const total   = stats.total
      const info    = `${total}`
      const type    = (total < 2 ? 'low' : (total < 4 ? 'medium' : 'high'))
      elements.push({ time, title, total, info, type })
    }
    return elements
  }

  renderDay(data) {
    return (<div key={data.dayId} styleName="dayContainer">
      <h2> { data.text } </h2>
      <div styleName="day">
      { this.hours(data).map(hour => this.renderHour(data, hour)) }
    </div></div>)
  }

  renderDays() {
    var days = []
    for(var dayId = 0; dayId <= this.state.daysLeft; dayId++) {
      const day = moment().add(dayId, 'days')
      const text = day.format("dddd, Do MMMM")
      days.push(this.renderDay({ day, dayId, text }))
    }
    return days
  }

  render () {
    if (this.state.sending) {
      return this.renderProgress()
    }

    if (this.state.signupActive) {
      return this.renderSignup()
    }
    return (
        <div styleName="container">
          <h1> { this.props.carmel.text.title } </h1>
          { this.renderDays() }
        </div>
    )
  }
}

export default CSSModules(CarmelComponent, style);
